/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.gwt.jsf;

import java.lang.reflect.Method;

import javax.el.MethodExpression;

import org.ajax4jsf.gwt.client.GwtFacesEvent;
import org.ajax4jsf.gwt.client.GwtFacesResult;

import com.sun.facelets.FaceletContext;
import com.sun.facelets.tag.MetaRule;
import com.sun.facelets.tag.Metadata;
import com.sun.facelets.tag.MetadataTarget;
import com.sun.facelets.tag.TagAttribute;

/**
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.4 $ $Date: 2006/07/13 19:54:16 $
 *
 */
public class GwtActionsRule extends MetaRule {


    public final static Class[] GWT_LISTENER_SIG = new Class[] { GwtFacesEvent.class };


    public final static class GwtListenerMapper extends Metadata {

        private final TagAttribute attr;
        
        private final String name;

        public GwtListenerMapper(String name, TagAttribute attr) {
            this.attr = attr;
            // convert attribute name to setter
            this.name = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
        }

        public void applyMetadata(FaceletContext ctx, Object instance) {
			MethodExpression methodExpression = this.attr.getMethodExpression(ctx, GwtFacesResult.class,GwtActionsRule.GWT_LISTENER_SIG);
        	if (instance instanceof GwtSource) {
				GwtSource gwtSource = (GwtSource) instance;
				gwtSource.addGwtListener(new GwtListenerMethodHelper(methodExpression,null));
			} else {
        	Class clazz = instance.getClass();
        	try {
        		// Find setter method
				Method setter = clazz.getMethod(name,
						new Class[] { MethodExpression.class });
				// Set gwtActionListener method expression
				setter.invoke(instance, new Object[] { methodExpression });
			} catch (Exception e) {
				// TODO: handle exception
			}
			}
        }

    }

    public final static GwtActionsRule instance = new GwtActionsRule();

    public GwtActionsRule() {
        super();
    }

    public Metadata applyRule(String name, TagAttribute attribute,
            MetadataTarget meta) {
        if (meta.isTargetInstanceOf(GwtComponent.class)) {

            if (name.startsWith("gwt") && name.endsWith("Listener")) {
                    return new GwtListenerMapper(name,attribute);
            }
        }
        return null;
    }

}
