/**
 * 
 */
package org.ajax4jsf.gwt.jsf;

import java.io.IOException;

import javax.el.ELException;
import javax.el.MethodExpression;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;

import org.ajax4jsf.gwt.client.GwtFacesResult;

import com.sun.facelets.FaceletContext;
import com.sun.facelets.FaceletException;
import com.sun.facelets.tag.TagAttribute;
import com.sun.facelets.tag.TagAttributeException;
import com.sun.facelets.tag.TagConfig;
import com.sun.facelets.tag.TagException;
import com.sun.facelets.tag.TagHandler;

/**
 * @author shura
 *
 */
public class GwtListenerHandler extends TagHandler {

    private Class listenerType;
    
    private Class[] signature = GwtActionsRule.GWT_LISTENER_SIG; 

    private final TagAttribute type;

    private final TagAttribute binding;

    private final TagAttribute method;

    private final TagAttribute event;

    private Class eventType;

    public GwtListenerHandler(TagConfig config) {
		super(config);
        this.binding = this.getAttribute("binding");
        this.method = this.getAttribute("method");
        this.event = this.getAttribute("event");
        this.type = this.getAttribute("type");
        if (type != null) {
            if (!type.isLiteral()) {
                throw new TagAttributeException(this.tag, this.type, "Must be literal");
            }
            try {
                this.listenerType = Class.forName(type.getValue());
            } catch (Exception e) {
                throw new TagAttributeException(this.tag, this.type, e);
            }
        }
        if (event != null) {
            if (!event.isLiteral()) {
                throw new TagAttributeException(this.tag, this.event, "Must be literal");
            }
            try {
                this.eventType = Class.forName(event.getValue());
            	this.signature = new Class[]{this.eventType};
            } catch (Exception e) {
                throw new TagAttributeException(this.tag, this.event, e);
            }
        }
	}

	/* (non-Javadoc)
	 * @see com.sun.facelets.FaceletHandler#apply(com.sun.facelets.FaceletContext, javax.faces.component.UIComponent)
	 */
	public void apply(FaceletContext ctx, UIComponent parent)
			throws IOException, FacesException, FaceletException, ELException {
        if (parent instanceof GwtSource) {
            // only process if parent was just created
            if (parent.getParent() == null) {
                GwtSource src = (GwtSource) parent;
                GwtListener listener = null;
                ValueExpression ve = null;
                if (this.binding != null) {
                    ve = this.binding.getValueExpression(ctx,
                            GwtListener.class);
                    listener = new GwtListenerHelper(ve);
                } else if (this.method != null) {
					MethodExpression me = this.method.getMethodExpression(ctx,GwtFacesResult.class,signature);
                    listener = new GwtListenerMethodHelper(me,eventType);
				}
                if (listener == null) {
                    try {
                        listener = (GwtListener) listenerType.newInstance();
                    } catch (Exception e) {
                        throw new TagAttributeException(this.tag, this.type, e.getCause());
                    }
                    if (ve != null) {
                        ve.setValue(ctx, ve);
                    }
                }
                src.addGwtListener(listener);
            }
        } else {
            throw new TagException(this.tag,
                    "Parent is not of type GwtSource, type is: " + parent);
        }
	}

}
