/**
 * 
 */
package org.ajax4jsf.gwt.server;

import java.lang.reflect.Method;
import java.util.Enumeration;

import javax.faces.component.ContextCallback;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseListener;
import javax.faces.lifecycle.Lifecycle;
import javax.servlet.http.HttpServletRequest;

import org.ajax4jsf.gwt.client.ComponentEntryPoint;
import org.ajax4jsf.gwt.client.GwtFacesEvent;
import org.ajax4jsf.gwt.client.GwtFacesException;
import org.ajax4jsf.gwt.client.GwtFacesResult;
import org.ajax4jsf.gwt.client.GwtFacesService;
import org.ajax4jsf.gwt.jsf.GwtComponent;
import org.ajax4jsf.gwt.jsf.GwtCallListener;
import org.ajax4jsf.gwt.jsf.GwtEvent;
import org.ajax4jsf.gwt.jsf.GwtListener;
import org.ajax4jsf.gwt.jsf.GwtSource;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * @author shura
 * 
 */
public class GwtFacesServlet extends RemoteServiceServlet implements
		GwtFacesService {

	protected Lifecycle lifecycle;
	
	private static final Class[] CALL_PARAMS = {GwtFacesEvent.class};


	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
	 */
//	public void init(ServletConfig config) throws ServletException {
//		super.init(config);
//		try {
//			LifecycleFactory factory;
//			factory = (LifecycleFactory) FactoryFinder
//					.getFactory(FactoryFinder.LIFECYCLE_FACTORY);
//			lifecycle = factory
//					.getLifecycle(LifecycleFactory.DEFAULT_LIFECYCLE);
//		} catch (Exception e) {
//			 
//		}
//	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.gwt.sample.client.GwtFacesService#sendEvent(org.ajax4jsf.gwt.sample.client.GwtFacesEvent)
	 */
	public GwtFacesResult sendEvent(final GwtFacesEvent event)
			throws GwtFacesException {
		FacesContext facesContext = FacesContext.getCurrentInstance();
		String clientId ;
		GwtFacesResult gwtFacesResult = null;
		if (null != facesContext) {
			clientId = (String) facesContext.getExternalContext().getRequestParameterMap().get("clientId");
			if (null != clientId) {
				// Invoke event on target component
				GwtFacesCallback callback = new GwtFacesCallback(event);
				try {
					facesContext.getViewRoot().invokeOnComponent(facesContext,
							clientId, callback);
				} catch (Exception e) {
					throw new GwtFacesException(
							"Error send event to component: " + e.getMessage());
				}
				gwtFacesResult = callback.getGwtFacesResult();
			} else if(null != lifecycle){
				// Invoke event on registered listeners.
				PhaseListener[] listeners = lifecycle.getPhaseListeners();
				for (int i = 0; i < listeners.length; i++) {
					PhaseListener listener = listeners[i];
					if (listener instanceof GwtCallListener) {
						GwtCallListener gwtListener = (GwtCallListener) listener;
						gwtFacesResult = gwtListener.processGwtEvent(event);
					}
				}
			}
		} else {
			// Non-faces environment, attempt to load stub class for hosted mode
			try {
				HttpServletRequest threadLocalRequest = getThreadLocalRequest();
				String moduleName = threadLocalRequest.getParameter(ComponentEntryPoint.GWT_MODULE_NAME_PARAMETER);
				if(null == moduleName){
					throw new GwtFacesException("Module name not set in request");
				}
				// Find latest package delimiter. GWT not allow to use default package - since
				// always present.
				int i = moduleName.lastIndexOf('.');
				// Module class name
				String className = "Mock" + moduleName.substring(i+1);
				// Module package name
				String packageName = moduleName.substring(0,i)+".server.";
				ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
				className = packageName+className;
				Class clazz = classLoader.loadClass(className);
				GwtFacesService service = (GwtFacesService) clazz.newInstance();
				gwtFacesResult = service.sendEvent(event);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				throw new GwtFacesException("Error to call test method"+e.getMessage());
			}
		}
		return gwtFacesResult;
	}

	private static class GwtFacesCallback implements ContextCallback {
		private GwtFacesResult gwtFacesResult = null;

		private GwtFacesEvent event;

		/**
		 * @param event
		 */
		public GwtFacesCallback(GwtFacesEvent event) {
			super();
			// TODO Auto-generated constructor stub
			this.event = event;
		}

		public void invokeContextCallback(FacesContext context,
				UIComponent target) {
			if (target instanceof GwtSource) {
				GwtEvent gwtEvent = new GwtEvent(target){

					public GwtFacesResult getCallbackParameter() {
						// TODO Auto-generated method stub
						return gwtFacesResult;
					}

					public void setCallbackParameter(GwtFacesResult callbackParam) {
						gwtFacesResult = callbackParam;
						
					}

					public GwtFacesEvent getGwtEventData() {
						// TODO Auto-generated method stub
						return event;
					}
					
				};
				target.broadcast(gwtEvent);
			}
			if (target instanceof GwtListener) {
				GwtListener listener = (GwtListener) target;
				if(listener.isAcceptEvent(event)){
					gwtFacesResult = listener.processEvent(event);
				}
			}

		}

		/**
		 * @return Returns the gwtFacesResult.
		 */
		public GwtFacesResult getGwtFacesResult() {
			return gwtFacesResult;
		}

	}
}
